==> kreetingkard-0.4.2.tar.gz: kreetingkard/src/kkitem.cpp <==
/***************************************************************************
 *   Copyright (C) 2004 by Tasuku Suzuki                                   *
 *   tasuku@linux-life.net                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qregexp.h>
#include <qimage.h>
#include <qcstring.h>
#include <qpixmap.h>
#include <qdatastream.h>
#include <qvaluelist.h>

#include <kiconloader.h>
#include <kmdcodec.h>
#include <klocale.h>
#include <kdebug.h>

#include "address.h"
#include "kkitem.h"

KKItem::KKItem( KListViewItem* parent, const char* name )
	: QObject( (QWidget*)0, name ), KListViewItem( parent )
{
	setPixmap( 0, SmallIcon("frame_edit") );
	setText( 0, i18n( "Item" ) );
	m_text = "";
	m_textAlign = AlignCenter;
	m_textOrientation = Horizontal;
	m_textColor = black;
	m_textFont = listView()->font();
	m_imagePrint = true;
	m_isBase = false;
	m_datafield = 0;
}

KKItem::KKItem( QDomElement doc, KListViewItem* parent, const char* name )
	: QObject( (QWidget*)0, name ), KListViewItem( parent )
{
	setPixmap( 0, SmallIcon("frame_edit") );
	m_isBase = false;
	parse( doc );
}

KKItem::KKItem( QDomElement doc, KListView* parent, const char* name )
	: QObject( (QWidget*)0, name ), KListViewItem( parent )
{
	setPixmap( 0, SmallIcon("filenew") );
	m_isBase = true;
	parse( doc );
}

void KKItem::parse( QDomElement& e )
{
	m_selected = false;

	setText( 0, e.attribute( "name" ) );
	m_location.setX( e.hasAttribute( "x" ) ? e.attribute( "x" ).toInt() : 0 );
	m_location.setY( e.hasAttribute( "y" ) ? e.attribute( "y" ).toInt() : 0 );

	m_size.setWidth( e.hasAttribute( "width" ) ? e.attribute( "width" ).toInt() : 10 );
	m_size.setHeight( e.hasAttribute( "height" ) ? e.attribute( "height" ).toInt() : 10 );

	m_text = e.hasAttribute( "text" ) ? e.attribute( "text" ) : "";
	m_textOrientation = e.hasAttribute( "textorientation" ) ? ((Orientation)e.attribute( "textorientation" ).toInt()) : Horizontal;
	m_textAlign = e.hasAttribute( "textalign" ) ? e.attribute( "textalign" ).toInt() : AlignCenter;
	m_textColor = e.hasAttribute( "textcolor" ) ? QColor( e.attribute( "textcolor" ) ) : black;
	if( e.hasAttribute( "textfont" ) && !e.attribute( "textfont" ).isEmpty() )
	{
		QFont f;
		if( f.fromString( e.attribute("textfont" ) ) )
		{
			m_textFont = f;
		}
	}
	else
	{
		m_textFont = listView()->font();
	}

	m_datafield = e.hasAttribute( "datafield" ) ? e.attribute( "datafield" ).toInt() : 0;

	if( e.hasAttribute( "image" ) && !e.attribute( "image" ).isEmpty() )
	{
		QByteArray borg, bdec;
		borg = e.attribute( "image" ).utf8();
		KCodecs::base64Decode( borg, bdec );
		QDataStream stream( bdec, IO_ReadOnly );
		stream >> m_image;
	}

	m_imagePrint = e.hasAttribute( "imageprint" );

	listView()->setOpen( this, e.hasAttribute( "selected" ) );
	listView()->setSelected( this, e.hasAttribute( "selected" ) );

	for( QDomNode n = e.lastChild(); !n.isNull(); n = n.previousSibling() )
	{
		switch( n.nodeType() )
		{
			case QDomNode::ElementNode:
				new KKItem( n.toElement(), this );
				break;
			default:
				break;
		}
	}
}

KKItem::~KKItem()
{
}

// BEGIN Properties

// BEGIN Name
void KKItem::setName( QString n )
{
	if( n == name() ) return;
	setText( 0, n );
	emit nameChanged();
}
// END

// BEGIN Location
void KKItem::setLocation( QPoint l )
{
	if( l == location() ) return;
	QRect r = rect();
	m_location = l;
	emit repaint( rect() );
	emit repaint( r );
	emit locationChanged();
}
// END

// BEGIN Size
void KKItem::setSize( QSize s )
{
	if( s == size() ) return;
	QRect r = rect();
	m_size = s;
	emit repaint( rect() );
	emit repaint( r );
	emit sizeChanged();
}
// END

// BEGIN Text
void KKItem::setText( QString t )
{
	if( t == text() ) return;
	m_text = t;
	emit repaint( rect() );
	emit textChanged();
}
// END

// BEGIN TextAlign
void KKItem::setTextAlign( unsigned int a )
{
	if( a == textAlign() ) return;
	m_textAlign = a;
	emit repaint( rect() );
	emit textAlignChanged();
}
// END

// BEGIN TextOrientation
void KKItem::setTextOrientation( Orientation o )
{
	if( o == textOrientation() ) return;
	m_textOrientation = o;
	emit repaint( rect() );
	emit textOrientationChanged();
}
// END

// BEGIN TextColor
void KKItem::setTextColor( QColor c )
{
	if( c == textColor() ) return;
	m_textColor = c;
	emit repaint( rect() );
	emit textColorChanged();
}
// END

// BEGIN Font
void KKItem::setTextFont( QFont f )
{
	if( f == textFont() ) return;
	m_textFont = f;
	emit repaint( rect() );
	emit textFontChanged();
}
// END

// BEGIN DataField
void KKItem::setDataField( int d )
{
	if( d == datafield() ) return;
	m_datafield = d;
	emit datafieldChanged();
}

// END DataField

// BEGIN Image
void KKItem::setImage( QPixmap i )
{
	if( i.convertToImage() == image().convertToImage() ) return;
	m_image = i;
	emit repaint( rect() );
	emit imageChanged();
}
// END

// BEGIN Orientation
void KKItem::setOrientation( Orientation o )
{
	if( o == orientation() ) return;
	QSize s = size();
	switch( o )
	{
		case Horizontal:
			if( s.width() < s.height() )
			{
				m_size = QSize( s.height(), s.width() );
			}
			break;
		case Vertical:
			if( s.width() > s.height() )
			{
				m_size = QSize( s.height(), s.width() );
			}
			break;
	}
	emit repaint( rect() );
	emit orientationChanged();
}
// END

// BEGIN Print
void KKItem::setImagePrint( bool p )
{
	m_imagePrint = p;
	emit imagePrintChanged();
}
// END

// END   Properties

void KKItem::setSelected( bool b )
{
	KListViewItem::setSelected( b );
	emit selectionChanged( b );
	emit selectionChanged( this, b );
	emit repaint( rect() );
}

void KKItem::drawItem( QPainter& p, double zoom, Address* address )
{
	p.setPen( isSelected() ? blue : gray );
	QRect r( location()*zoom, size()*zoom );

	if( !address )
	{
		p.drawLine( r.topLeft(), r.topRight() );
		p.drawLine( r.topRight(), r.bottomRight() );
		p.drawLine( r.bottomRight(), r.bottomLeft() );
		p.drawLine( r.bottomLeft(), r.topLeft() );
		r.setRect( r.left() + 1, r.top() + 1, r.width() - 2, r.height() - 2 );
	}
	if( !image().isNull() )
	{
		if( address )
		{
			// when printing
			if( imagePrint() )
			{
				p.drawPixmap( r, image() );
			}
		}
		else
		{
			// when editing,previewing
			if( QListViewItem::parent() )
			{
				p.drawPixmap( r, image() );
			}
		}
	}
	if( address && datafield() )
	{
		switch( datafield() )
		{
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
				if( !address->zipcode[datafield()-1].isNull() )
				{
					drawText( p, r, QString( address->zipcode[datafield()-1] ) );
				}
				break;
			case 8:	// Direction
				if( !address->direction.isEmpty() )
				{
					drawText( p, r, address->direction );
				}
				break;
			case 9:	// Address
				if( !address->address.isEmpty() )
				{
					drawText( p, r, address->address );
				}
				break;
			case 10:	// Comment
				if( !address->comment.isEmpty() )
				{
					drawText( p, r, address->comment );
				}
				break;
		}
	}
	else if( !text().isEmpty() )
	{
		drawText( p, r, text() );
	}
}

void KKItem::drawText( QPainter& p, QRect r, QString text )
{
	p.setPen( textColor() );
	p.setFont( textFont() );
	if( textOrientation() == Horizontal )
	{
		p.drawText( r, textAlign(), text );
		return;
	}
	QStringList org = QStringList::split( "\n", text, true );
	QStringList disp;
	for( QStringList::Iterator it = org.begin(); it != org.end(); ++it )
	{
		QStringList line = QStringList::split( QString(""), *it, true );
		line.pop_front();
		line.pop_back();
		QString tmp = line.join( "\n" );

		QRegExp hyphen( QString::fromUtf8("[-ーｰ－-]") );
		tmp.replace( hyphen, "|" );

		QRegExp rx( "([0-9])\n([0-9])" );
		while( rx.search( tmp ) != -1 )
		{
			tmp.replace( rx, "\\1\\2" );
		}
		disp.append( tmp );
	}

	int* width = new int[disp.count()];
	QFontMetrics met( textFont() );
	for( unsigned int i = 0; i != disp.count(); i++ )
	{
		width[i] = 0;
		QStringList line = QStringList::split( "\n", disp[i], true );
		for( QStringList::Iterator it = line.begin(); it != line.end(); ++it )
		{
			if( width[i] < met.width( *it ) )
			{
				width[i] = met.width( *it );
			}
		}
	}
	int right = r.width();
	for( unsigned int i = 0; i != disp.count(); i++ )
	{
		right -= width[i];
	}
	switch( textAlign() & AlignHorizontal_Mask )
	{
		case AlignLeft:
			right = r.right() - right;
			break;
		case AlignHCenter:
			right = r.right() - right / 2;
			break;
		case AlignRight:
			right = r.right();
			break;
	}
	for( unsigned int i = 0; i != disp.count(); i++ )
	{
		QRect rct( right - width[i], r.top(), width[i], r.height() );
		if( r.contains( rct ) )
		{
			p.drawText( rct, AlignHCenter | ( textAlign() & AlignVertical_Mask ), disp[i] );
		}
		right -= width[i];
	}
	delete[] width;
}

QDomElement KKItem::toElement( QDomDocument& doc )
{
	QDomElement elem = doc.createElement( "item" );
	if( !QListViewItem::parent() )
	{
		KKItem* child = (KKItem*)firstChild();
		while( child )
		{
			elem.appendChild( child->toElement( doc ) );
			child = (KKItem*)child->nextSibling();
		}
		elem.setAttribute( "orientation", orientation() );
	}
	else
	{
		elem.setAttribute( "x", location().x() );
		elem.setAttribute( "y", location().y() );
		elem.setAttribute( "text", text() );
		elem.setAttribute( "textalign", textAlign() );
		elem.setAttribute( "textorientation", textOrientation() );
		elem.setAttribute( "textcolor", textColor().name() );
		elem.setAttribute( "textfont", textFont().toString() );

		elem.setAttribute( "datafield", datafield() );
	}
	elem.setAttribute( "name", text(0) );
	elem.setAttribute( "width", size().width() );
	elem.setAttribute( "height", size().height() );

	if( !image().isNull() )
	{
		QByteArray borg, benc;
		QDataStream stream( borg, IO_WriteOnly );
		stream << image();
		KCodecs::base64Encode( borg, benc );
		elem.setAttribute( "image", benc );
	}
	if( m_imagePrint ) elem.setAttribute( "imageprint", 1 );
	if( isSelected() ) elem.setAttribute( "selected", 1 );
	return elem;
}

void KKItem::setExist( bool bl )
{
	if( bl )
	{
		emit inserted( this );
	}
	else
	{
		emit removed( this );
	}
}

==> kreetingkard-0.4.3.tar.gz: kreetingkard/src/kkitem.cpp <==
/***************************************************************************
 *   Copyright (C) 2004 by Tasuku Suzuki                                   *
 *   tasuku@linux-life.net                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qregexp.h>
#include <qimage.h>
#include <qcstring.h>
#include <qpixmap.h>
#include <qdatastream.h>
#include <qvaluelist.h>

#include <kiconloader.h>
#include <kmdcodec.h>
#include <klocale.h>
#include <kdebug.h>

#include "address.h"
#include "kkitem.h"

KKItem::KKItem( QListViewItem* parent, KKItem* item )
	: QObject( (QWidget*)0 ), KListViewItem( parent )
{
	setPixmap( 0, SmallIcon("frame_edit") );
	setName( item->name() );
	setLocation( item->location() );
	setSize( item->size() );
	setText( item->text() );
	setTextAlign( item->textAlign() );
	setTextOrientation( item->textOrientation() );
	setTextColor( item->textColor() );
	setTextFont( item->textFont() );
	if( !item->image().isNull() ) setImage( item->image() );
	setImagePrint( item->imagePrint() );
	m_isBase = false;
	m_datafield = 0;
}

KKItem::KKItem( QListViewItem* parent, const char* name )
	: QObject( (QWidget*)0, name ), KListViewItem( parent )
{
	setPixmap( 0, SmallIcon("frame_edit") );
	setText( 0, i18n( "Item" ) );
	m_text = "";
	m_textAlign = AlignCenter;
	m_textOrientation = Horizontal;
	m_textColor = black;
	m_textFont = listView()->font();
	m_imagePrint = true;
	m_isBase = false;
	m_datafield = 0;
}

KKItem::KKItem( QDomElement doc, KListViewItem* parent, const char* name )
	: QObject( (QWidget*)0, name ), KListViewItem( parent )
{
	setPixmap( 0, SmallIcon("frame_edit") );
	m_isBase = false;
	parse( doc );
}

KKItem::KKItem( QDomElement doc, KListView* parent, const char* name )
	: QObject( (QWidget*)0, name ), KListViewItem( parent )
{
	setPixmap( 0, SmallIcon("filenew") );
	m_isBase = true;
	parse( doc );
}

void KKItem::parse( QDomElement& e )
{
	m_selected = false;

	setText( 0, e.attribute( "name" ) );
	m_location.setX( e.hasAttribute( "x" ) ? e.attribute( "x" ).toInt() : 0 );
	m_location.setY( e.hasAttribute( "y" ) ? e.attribute( "y" ).toInt() : 0 );

	m_size.setWidth( e.hasAttribute( "width" ) ? e.attribute( "width" ).toInt() : 10 );
	m_size.setHeight( e.hasAttribute( "height" ) ? e.attribute( "height" ).toInt() : 10 );

	m_text = e.hasAttribute( "text" ) ? e.attribute( "text" ) : "";
	m_textOrientation = e.hasAttribute( "textorientation" ) ? ((Orientation)e.attribute( "textorientation" ).toInt()) : Horizontal;
	m_textAlign = e.hasAttribute( "textalign" ) ? e.attribute( "textalign" ).toInt() : AlignCenter;
	m_textColor = e.hasAttribute( "textcolor" ) ? QColor( e.attribute( "textcolor" ) ) : black;
	if( e.hasAttribute( "textfont" ) && !e.attribute( "textfont" ).isEmpty() )
	{
		QFont f;
		if( f.fromString( e.attribute("textfont" ) ) )
		{
			m_textFont = f;
		}
	}
	else
	{
		m_textFont = listView()->font();
	}

	m_datafield = e.hasAttribute( "datafield" ) ? e.attribute( "datafield" ).toInt() : 0;

	if( e.hasAttribute( "image" ) && !e.attribute( "image" ).isEmpty() )
	{
		QByteArray borg, bdec;
		borg = e.attribute( "image" ).utf8();
		KCodecs::base64Decode( borg, bdec );
		QDataStream stream( bdec, IO_ReadOnly );
		stream >> m_image;
		m_imageresized.convertFromImage( m_image.convertToImage().smoothScale( size() ) );
	}

	m_imagePrint = e.hasAttribute( "imageprint" );

	listView()->setOpen( this, e.hasAttribute( "selected" ) );
	listView()->setSelected( this, e.hasAttribute( "selected" ) );

	for( QDomNode n = e.lastChild(); !n.isNull(); n = n.previousSibling() )
	{
		switch( n.nodeType() )
		{
			case QDomNode::ElementNode:
				new KKItem( n.toElement(), this );
				break;
			default:
				break;
		}
	}
}

KKItem::~KKItem()
{
}

// BEGIN Properties

// BEGIN Name
void KKItem::setName( const QString &n )
{
	if( n == name() ) return;
	setText( 0, n );
	emit nameChanged();
}
// END

// BEGIN Location
void KKItem::setLocation( const QPoint &l )
{
	if( l == location() ) return;
	QRect r = rect();
	m_location = l;
	emit repaint( r | rect() );
	emit locationChanged();
}
// END

// BEGIN Size
void KKItem::setSize( const QSize &s )
{
	if( s == size() ) return;
	QRect r = rect();
	m_size = s;
	if( !image().isNull() )
	{
		m_imageresized.convertFromImage( image().convertToImage().smoothScale( size() ) );
	}
	emit repaint( r | rect() );
	emit sizeChanged();
}
// END

// BEGIN Text
void KKItem::setText( const QString &t )
{
	if( t == text() ) return;
	m_text = t;
	emit repaint( rect() );
	emit textChanged();
}
// END

// BEGIN TextAlign
void KKItem::setTextAlign( unsigned int a )
{
	if( a == textAlign() ) return;
	m_textAlign = a;
	emit repaint( rect() );
	emit textAlignChanged();
}
// END

// BEGIN TextOrientation
void KKItem::setTextOrientation( Orientation o )
{
	if( o == textOrientation() ) return;
	m_textOrientation = o;
	emit repaint( rect() );
	emit textOrientationChanged();
}
// END

// BEGIN TextColor
void KKItem::setTextColor( const QColor &c )
{
	if( c == textColor() ) return;
	m_textColor = c;
	emit repaint( rect() );
	emit textColorChanged();
}
// END

// BEGIN Font
void KKItem::setTextFont( const QFont &f )
{
	if( f == textFont() ) return;
	m_textFont = f;
	emit repaint( rect() );
	emit textFontChanged();
}
// END

// BEGIN DataField
void KKItem::setDataField( int d )
{
	if( d == datafield() ) return;
	m_datafield = d;
	emit datafieldChanged();
}

// END DataField

// BEGIN Image
void KKItem::setImage( const QPixmap &i )
{
	if( i.convertToImage() == image().convertToImage() ) return;
	m_image = i;
	m_imageresized.convertFromImage( m_image.convertToImage().smoothScale( size() ) );
	emit repaint( rect() );
	emit imageChanged();
}
// END

// BEGIN Orientation
void KKItem::setOrientation( Orientation o )
{
	if( o == orientation() ) return;
	QSize s = size();
	switch( o )
	{
		case Horizontal:
			if( s.width() < s.height() )
			{
				m_size = QSize( s.height(), s.width() );
			}
			break;
		case Vertical:
			if( s.width() > s.height() )
			{
				m_size = QSize( s.height(), s.width() );
			}
			break;
	}
	emit repaint( rect() );
	emit orientationChanged();
}
// END

// BEGIN Print
void KKItem::setImagePrint( bool p )
{
	m_imagePrint = p;
	emit imagePrintChanged();
}
// END

// END   Properties

void KKItem::setSelected( bool b )
{
	KListViewItem::setSelected( b );
	emit selectionChanged( b );
	emit selectionChanged( this, b );
	emit repaint( rect() );
}

void KKItem::drawItem( QPainter& p, double zoom, Address* address )
{
	p.setPen( isSelected() ? blue : gray );
	QRect r( location()*zoom, size()*zoom );

	if( !address )
	{
		p.drawLine( r.topLeft(), r.topRight() );
		p.drawLine( r.topRight(), r.bottomRight() );
		p.drawLine( r.bottomRight(), r.bottomLeft() );
		p.drawLine( r.bottomLeft(), r.topLeft() );
		r.setRect( r.left() + 1, r.top() + 1, r.width() - 2, r.height() - 2 );
	}
	if( !image().isNull() )
	{
		if( address )
		{
			// when printing
			if( imagePrint() )
			{
				p.drawPixmap( r, imageResized() );
			}
		}
		else
		{
			// when editing,previewing
			if( QListViewItem::parent() )
			{
				p.drawPixmap( r, imageResized() );
			}
		}
	}
	if( address && datafield() )
	{
		switch( datafield() )
		{
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
				if( !address->zipcode[datafield()-1].isNull() )
				{
					drawText( p, r, QString( address->zipcode[datafield()-1] ) );
				}
				break;
			case 8:	// Direction
				if( !address->direction.isEmpty() )
				{
					drawText( p, r, address->direction );
				}
				break;
			case 9:	// Address
				if( !address->address.isEmpty() )
				{
					drawText( p, r, address->address );
				}
				break;
			case 10:	// Comment
				if( !address->comment.isEmpty() )
				{
					drawText( p, r, address->comment );
				}
				break;
		}
	}
	else if( !text().isEmpty() )
	{
		drawText( p, r, text() );
	}
}

void KKItem::drawText( QPainter& p, QRect r, const QString &text )
{
	p.setPen( textColor() );
	p.setFont( textFont() );
	if( textOrientation() == Horizontal )
	{
		p.drawText( r, textAlign(), text );
		return;
	}
	QStringList org = QStringList::split( "\n", text, true );
	QStringList disp;
	for( QStringList::Iterator it = org.begin(); it != org.end(); ++it )
	{
		QStringList line = QStringList::split( QString(""), *it, true );
		line.pop_front();
		line.pop_back();
		QString tmp = line.join( "\n" );

		QRegExp hyphen( QString::fromUtf8("[-ーｰ－-]") );
		tmp.replace( hyphen, "|" );

		QRegExp rx( "([0-9])\n([0-9])" );
		while( rx.search( tmp ) != -1 )
		{
			tmp.replace( rx, "\\1\\2" );
		}
		disp.append( tmp );
	}

	int* width = new int[disp.count()];
	QFontMetrics met( textFont() );
	for( unsigned int i = 0; i != disp.count(); i++ )
	{
		width[i] = 0;
		QStringList line = QStringList::split( "\n", disp[i], true );
		for( QStringList::Iterator it = line.begin(); it != line.end(); ++it )
		{
			if( width[i] < met.width( *it ) )
			{
				width[i] = met.width( *it );
			}
		}
	}
	int right = r.width();
	for( unsigned int i = 0; i != disp.count(); i++ )
	{
		right -= width[i];
	}
	switch( textAlign() & AlignHorizontal_Mask )
	{
		case AlignLeft:
			right = r.right() - right;
			break;
		case AlignHCenter:
			right = r.right() - right / 2;
			break;
		case AlignRight:
			right = r.right();
			break;
	}
	for( unsigned int i = 0; i != disp.count(); i++ )
	{
		QRect rct( right - width[i], r.top(), width[i], r.height() );
		if( r.contains( rct ) )
		{
			p.drawText( rct, AlignHCenter | ( textAlign() & AlignVertical_Mask ), disp[i] );
		}
		right -= width[i];
	}
	delete[] width;
}

QDomElement KKItem::toElement( QDomDocument& doc )
{
	QDomElement elem = doc.createElement( "item" );
	if( !QListViewItem::parent() )
	{
		KKItem* child = (KKItem*)firstChild();
		while( child )
		{
			elem.appendChild( child->toElement( doc ) );
			child = (KKItem*)child->nextSibling();
		}
		elem.setAttribute( "orientation", orientation() );
	}
	else
	{
		elem.setAttribute( "x", location().x() );
		elem.setAttribute( "y", location().y() );
		elem.setAttribute( "text", text() );
		elem.setAttribute( "textalign", textAlign() );
		elem.setAttribute( "textorientation", textOrientation() );
		elem.setAttribute( "textcolor", textColor().name() );
		elem.setAttribute( "textfont", textFont().toString() );

		elem.setAttribute( "datafield", datafield() );
	}
	elem.setAttribute( "name", text(0) );
	elem.setAttribute( "width", size().width() );
	elem.setAttribute( "height", size().height() );

	if( !image().isNull() )
	{
		QByteArray borg, benc;
		QDataStream stream( borg, IO_WriteOnly );
		stream << image();
		KCodecs::base64Encode( borg, benc );
		elem.setAttribute( "image", benc );
	}
	if( m_imagePrint ) elem.setAttribute( "imageprint", 1 );
	if( isSelected() ) elem.setAttribute( "selected", 1 );
	return elem;
}
